// Copyright 2022 BytePlus Pte. Ltd.
// SPDX-License-Identifier: MIT

import 'bytertc_event_define.dart';

/// Range audio attenuation coefficient observer
class RTCRangeAudioObserver {
  /// Callback about the current attenuation coefficient of remote users.
  ///
  /// In manual subscription mode, when any user in the room calls [RTCRangeAudio.updatePosition] to update their position or [RTCRangeAudio.updateReceiveRange] to update their audio receiving range, the relative distance between this user and other users in the room will change, and the attenuation coefficient calculated accordingly will change as well, which will be notified to users.  <br>
  /// You can determine whether to subscribe to a remote user's audio stream based on the attenuation coefficient of that user shown in this callback.
  ///
  /// Notes: You will not receive this callback immediately after updating your location or audio receiving range. SDK calculates the attenuation coefficient every 500 ms, and only trigger this callback if the result of the calculation is different from the previous one, or if the result is the same but more than 3 seconds after the last calculation.
  OnRangeAudioInfoType? onRangeAudioInfo;

  RTCRangeAudioObserver({
    this.onRangeAudioInfo,
  });
}
